﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Diagnostics;
using System.ServiceProcess;
using System.Text;
using System.Threading.Tasks;
using System.IO.Ports;
using System.Configuration.Install;

namespace ZetronPagerMonitor
{
    public partial class ZetronMonitor : ServiceBase
    {

        // create our serial port and log file object for use later
        private string strFilePath = Properties.Settings.Default.LogFile;
        private SerialPort serialPort = new SerialPort(Properties.Settings.Default.Port, 9600, Parity.None, 8, StopBits.One);

        public ZetronMonitor()
        {
            // setup some definitions for this win service
            this.ServiceName = "Zetron Pager Monitor";
            this.CanHandlePowerEvent = false;
            this.CanHandleSessionChangeEvent = false;
            this.CanPauseAndContinue = false;
            this.CanShutdown = true;
            this.CanStop = true;

            InitializeComponent();
        }

        protected override void OnStart(string[] args)
        {
           // start the background worker thread
            this.backgroundWorker1.RunWorkerAsync();
           
        }

        protected override void OnStop()
        {
            // stop the background worker and clean up any unmanaged objects
            backgroundWorker1.CancelAsync();
            serialPort.Close();
            
        }

        // called when system shutdown is happening
        protected override void OnShutdown()
        {
            backgroundWorker1.CancelAsync();
            serialPort.Close();
        }

        private void backgroundWorker1_DoWork(object sender, DoWorkEventArgs e)
        {
            serialPort.DataReceived += new SerialDataReceivedEventHandler(serialPort_DataReceived);
            serialPort.Open();
            while (true) { System.Threading.Thread.Sleep(5); };
        }

        private void serialPort_DataReceived(object sender, SerialDataReceivedEventArgs e)
        {
            SerialPort sp = (SerialPort)sender;
            string inData = sp.ReadLine();

            switch(inData)
            {
                case "HEARTBEAT":
                    WriteLogEntry(Properties.Settings.Default.LogFile, "Heartbeat Message");
                    break;
                case "PAGERECEIVED":
                    WriteLogEntry(Properties.Settings.Default.LogFile, "Page Received");
                    break;
                default:
                    WriteLogEntry(Properties.Settings.Default.LogFile, inData);
                    break;
            }

            // we are not doing anything with this progress report in the main thread but if we leave this here we can later
            backgroundWorker1.ReportProgress(0, inData);
        }


        private static void WriteLogEntry(string logFileLocation, string value)
        {
            // before we write to this log file, we need to see if we need to rotate the log file out
            // we currently rotate after 1.5MB filesize
            if (System.IO.File.Exists(@logFileLocation))
            {
                System.IO.FileInfo fileInfo = new System.IO.FileInfo(@logFileLocation);
                if (fileInfo.Length >= 1500000)
                {

                    fileInfo.MoveTo(System.IO.Path.GetDirectoryName(logFileLocation) + "\\winservicelog_" + System.DateTime.Now.ToString("mmddyyyyHHMM") + ".txt");
                }
            }
            
            using (System.IO.StreamWriter file = new System.IO.StreamWriter(@logFileLocation, true))
            {
                file.WriteLine(DateTime.Now + "| " + value);
            }
        }

    }
}
